; *****************************************************************************
; INDICE
; *****************************************************************************
; FUNCIONES
; -----------------------------------------------------------------------------
; int get_vesa_info()
; int get_mode_info(unsigned int mode)
; int find_vesa_mode(int wid, int hei, int depth)
; int set_vesa_mode(int mode)
; -----------------------------------------------------------------------------
;

%include "macros/v16_macros.mac"
BITS 32

extern vesa_info
extern mode_info

global get_vesa_info
global get_mode_info
global find_vesa_mode
global set_vesa_mode

es_val: dw 0
di_val: dw 0

; *****************************************************************************
; FUNCION get_vesa_info
; -----------------------------------------------------------------------------
; Descripcion
; -----------------------------------------------------------------------------
; 	Busca la info de VESA usando la INT 0x10 que proporciona la BIOS y la
; guarda en 'vesa_info'. Devuelve un int con 0 si todo salio bien, -1 en 
; caso contrario.
;
; -----------------------------------------------------------------------------
; Argumentos (-)
; -----------------------------------------------------------------------------
; Ejemplo de uso
; -----------------------------------------------------------------------------
; > if (get_vesa_info() == 0) {
; >    //be happy
; > } else {
; >    //be sad
; > }
marker: db 'VESA'
get_vesa_info:
	push ebp
	mov ebp, esp
	push ebx
	push esi
	push edi
	
	;Lleno con 0s
	mov edi, vesa_info
	xor ax, ax
	mov ecx, 256
	rep stosw
	
	mov eax, vesa_info
	shr eax, 4
	mov [es_val], ax
	
	mov eax, vesa_info
	and eax, 0xF
	mov [di_val], ax
	
	;Busco la info de VESA
	V16_INT 0x10, 'ax', 0x4F00, 'di', [di_val], 'es', [es_val]
	V16_INTRECOVER 'ax'
	
	test ah, ah		;ah = 0 => error!
	jnz .bad_ret
	
	mov eax, [vesa_info]
	mov ebx, [marker]
	cmp eax, ebx
	jne .bad_ret		;VESA != vesa_info.VESASignature => error!
	

	
.good_ret:
	xor eax, eax
	jmp .ret
	
.bad_ret:
	mov eax, -1
	
.ret:
	pop edi
	pop esi
	pop ebx
	pop ebp
	ret
	
; *****************************************************************************
; FUNCION get_mode_info
; -----------------------------------------------------------------------------
; Descripcion
; -----------------------------------------------------------------------------
; 	Busca la info de un modo de VESA usando la INT 0x10 que proporciona la 
; BIOS y la guarda en 'mode_info'. Devuelve un int con 0 si todo salio bien
; o bien con -1 si hubo algun problema.
;
; -----------------------------------------------------------------------------
; Argumentos (1)
; -----------------------------------------------------------------------------
; unsigned int mode = Modo de video del que se quiere obtener informacion
;
; Ejemplo de uso
; -----------------------------------------------------------------------------
; > get_mode_info(0);
; > if (mode_info.BitsPerPixel != 24) {
; > 		//old school!
; > }
;
%define gmi_mode ebp + 8
get_mode_info:
	push ebp
	mov ebp, esp
	push ebx
	push esi
	push edi
	
	;Lleno con 0s
	mov edi, mode_info
	xor ax, ax
	mov ecx, 128
	rep stosw
	
	mov eax, mode_info
	shr eax, 4
	mov [es_val], ax
	
	mov eax, mode_info
	and eax, 0xF
	mov [di_val], ax
	
	;Busco la info de VESA
	V16_INT 0x10, 'ax', 0x4F01, 'di', [di_val], 'es', [es_val], 'cx', [gmi_mode]
	V16_INTRECOVER 'ax'
	
	test ah, ah		;ah = 0 => error!
	jnz .bad_ret
	
.good_ret:
	xor eax, eax
	jmp .ret
	
.bad_ret:
	mov eax, -1
	
.ret:
	pop edi
	pop esi
	pop ebx
	pop ebp
	ret
	
	
; *****************************************************************************
; FUNCION find_vesa_mode
; -----------------------------------------------------------------------------
; Descripcion
; -----------------------------------------------------------------------------
; 	Busca de entre todos los modos soportados por la placa de video aquel
; que tenga la resolucion y profundidad de color pasados como argumento. 
; 	Devuelve un int con el identificador del modo, o bien 0 si no se encontro 
; ninguno que cumple lo pedido.
;
; -----------------------------------------------------------------------------
; Argumentos (3)
; -----------------------------------------------------------------------------
; int width = Ancho en pixels de la resolucion buscada.
; int height = Alto en pixels de la resolucion buscada.
; int depth = Profundidad del color en bits.
;
; Ejemplo de uso
; -----------------------------------------------------------------------------
; > int mode = find_vesa_mode(640, 480, 24);
; > if (mode == 0) {
; > 		//no hay un modo con esas cualidades
; > }
;
%define fvm_wid ebp + 8
%define fvm_hei ebp + 12
%define fvm_dep ebp + 16
%define mode_ModeAttributes mode_info + 0x00
%define vesa_VideoModePtr vesa_info + 0x0E
%define mode_XResolution mode_info + 0x12
%define mode_YResolution mode_info + 0x14
%define mode_BitsPerPixel mode_info + 0x19
find_vesa_mode:
	push ebp
	mov ebp, esp
	push ebx
	push esi
	push edi
	
	mov esi, [vesa_VideoModePtr]	;esi = [seg, addr]
	mov eax, esi					;eax = [seg, addr]
	xor si, si						;esi = [seg, 0]
	sub eax, esi					;eax = [0, addr]
	shr esi, 12						;esi = seg << 4
	add esi, eax					;esi = seg << 4 + addr
	
	.loop_modes:
		;Lee del area de memoria que tiene la informacion de los modos
		;hasta que se encuentre el valor 0xFFFF
		movzx ebx, word [esi]
		cmp bx, 0xFFFF
		je .no_mode
		
		;Busca la informacion del modo
		push ebx
		call get_mode_info	
		add esp, 4
		test ax, ax
		jnz .loop_continue		;if (get_mode_info(mode) != 0) continue;
		
		;Hay que chequear que el modo sea de color, grafico y que
		;tenga LFB
		mov ax, [mode_ModeAttributes]
		and ax, 0x99
		cmp ax, 0x99
		jne .loop_continue
		
		;Chequeo que tenga el ancho que piden
		mov ax, [mode_XResolution]
		cmp ax, [fvm_wid]
		jne .loop_continue
		
		;Chequeo que tenga el alto que piden
		mov ax, [mode_YResolution]
		cmp ax, [fvm_hei]
		jne .loop_continue
		
		;Chequeo que tenga la profundidad que piden
		mov al, [mode_BitsPerPixel]
		cmp al, [fvm_dep]
		jne .loop_continue
		
		;Paso todos los chequeos! Devuelvo este modo...
		mov eax, ebx
		jmp .ret
		
		.loop_continue:
			add esi, 2
			jmp .loop_modes
	
	
.no_mode:
	xor eax, eax
	
.ret:
	pop edi
	pop esi
	pop ebx
	pop ebp
	ret
	
	
; *****************************************************************************
; FUNCION set_vesa_mode
; -----------------------------------------------------------------------------
; Descripcion
; -----------------------------------------------------------------------------
; 	Cambia al modo de VESA indicado como argumento. Devuelve un int con 0 si
; todo salio bien o bien -1 si hubo algun problema.
; -----------------------------------------------------------------------------
; Argumentos (1)
; -----------------------------------------------------------------------------
; int mode = Modo grafico a setear
;
; Ejemplo de uso
; -----------------------------------------------------------------------------
; > if (set_vesa_mode(0x13) == 0) {
; > 		//cambiado!
; > }
;
%define svm_mode ebp + 8
set_vesa_mode:
	push ebp
	mov ebp, esp
	push ebx
	push esi
	push edi
	
	or word [svm_mode], 0x4000	;Linear Front Buffer
	V16_INT 0x10, 'ax', 0x4F02, 'bx', [svm_mode]
	V16_INTRECOVER 'ax'
	
	test ah, ah		;ah = 0 => error!
	jnz .bad_ret

	
.good_ret:
	xor eax, eax
	jmp .ret
	
.bad_ret:
	mov eax, -1
	
.ret:
	pop edi
	pop esi
	pop ebx
	pop ebp
	ret
